/*
Copyright 2008-2009 University of Cambridge
Copyright 2008-2009 University of Toronto
Copyright 2008-2009 University of California, Berkeley

Licensed under the Educational Community License (ECL), Version 2.0 or the New
BSD license. You may not use this file except in compliance with one these
Licenses.

You may obtain a copy of the ECL 2.0 License and BSD License at
https://source.fluidproject.org/svn/LICENSE.txt
*/

/*global jQuery*/
/*global fluid*/
/*global demo*/

var demo = demo || {};
(function ($, fluid) {

    /**
     * Initialize all rich text inline edit components present on the inline-edit 
     * demo.
     */
    var inlineRichTextEditSetup = function () {
        
        var editors = [];
        
        /**
         * Create cancel and save buttons for a rich inline editor.
         * @param {Object} editor 
         */
        var makeButtons = function (editor) {
            $(".save", editor.container).click(function(){
                editor.finish();
                return false;
            });

            $(".cancel", editor.container).click(function(){
                editor.cancel();
                return false;
            });
        };
        
        /**
         * Create cancel and save buttons for all rich text editors.
         * @param {Object} editors array of rich inline editors.
         */
        var makeAllButtons = function (editors) {
            while (editors.length > 0) {
                makeButtons(editors.pop());
            }
        };          
        
        /**
         * Tiny MCE rich inline text editor example. 
         */
        editors.push(
            fluid.inlineEdit.tinyMCE("#richEdit1", {
                tinyMCE: {
                        width: 1024,
                        theme: "advanced",
                        theme_advanced_toolbar_location : "top"
                    }, 
                componentDecorators: {
                    type: "fluid.undoDecorator"
                }
            })
        );
        
        /**
         * FCK Editor rich inline text editor example. 
         */
        editors.push(
            fluid.inlineEdit.FCKEditor("#richEdit2", {
                FCKEditor: {BasePath: "../../../../tests/manual-tests/lib/fckeditor/"},
                componentDecorators: {
                    type: "fluid.undoDecorator"
                }
            })
        );
        
        makeAllButtons(editors);
    };

        
    demo.initInlineEdit = function () {        
        inlineRichTextEditSetup();
    };    
})(jQuery, fluid);